#include "Global.h"
#include "Rat.h"
#include "LG.h"

#undef	NO_COORD_IMPROVEMENT		   /* switch off weight permutation */

#undef	TEST_PP				    /*  print some diagnostic info  */
#define	TEST_PD				   /*  check Poincare duality of PP */

extern FILE *inFILE, *outFILE;

typedef struct {Long x[POINT_Nmax][W_Nmax]; int N, np;}   AmbiPointList;
typedef struct {Long x[POLY_Dmax][W_Nmax]; int N, n;}     AmbiLatticeBasis;

Long Wperm_to_GLZ(Long *W, int *d, Long **G, int *P);
void WeightLatticeBasis(Weight *_w, AmbiLatticeBasis *_B);	
void WeightMakePoints(Weight *_W, AmbiPointList *_P);
void ChangeToTrianBasis(AmbiPointList*,	AmbiLatticeBasis *, PolyPointList *);

void AllocPoCoLi(PoCoLi *P);		      /* allocate e[P.A] and c[P.A] */
void Free_PoCoLi(PoCoLi *P);			/* free P.e and P.c */
void PolyProd(PoCoLi *A,PoCoLi *B,PoCoLi *AB);			/* AB = A*B */
int  BottomUpQuot(PoCoLi *N,PoCoLi *D,PoCoLi *Q,PoCoLi *R);    /* Q*D = N-R */
void PrintPoCoLi(PoCoLi *P);
void UnitPoly(PoCoLi *P);
void Init1_xN(PoCoLi *P,int N);					 /* 1 - x^N */
void PoincarePoly(int N,int *w,int d, PoCoLi *PP, PoCoLi *Naux, PoCoLi *Raux);

/*  ==========  	  I/O functions:                	==========  */

int  IsDigit(char c) { return (('0'<=c) && (c<='9'));}
void Write_Weight(Weight *_W)
{    int n; fprintf(outFILE,"%d ",(int) _W->d);
     for(n=0; n < _W->N; n++) fprintf(outFILE," %d",(int) _W->w[n]); 
     fprintf(outFILE,"\n");     /* puts("  = d  w_1 ... w_N"); */
}
int  Read_Weight(Weight *_W)     /* read "d w_i" [ or "w_i d" if last=max ] */
{    char c; int i, shift=1, I[W_Nmax+2], FilterFlag=(inFILE==NULL); 
     if(FilterFlag) inFILE=stdin; else
     if(inFILE==stdin) printf("type degree and weights  [d  w1 w2 ...]: "); 
     c=fgetc(inFILE); if( !IsDigit(c) ) return 0; ungetc(c,inFILE); 
     fscanf(inFILE,"%d",I);
     for(i=1; i<W_Nmax+2; i++)
     {	while(' '==(c=fgetc(inFILE))); ungetc(c,inFILE);
	if(IsDigit(c)) fscanf(inFILE,"%d",&I[i]); else break;
     }  while(fgetc(inFILE )-'\n') if(feof(inFILE))return 0; /* read to EOL */
     _W->N=i-1; if(_W->N>W_Nmax) { puts("Increase POLY_Dmax"); exit(0); }
     for(i=0;i<=_W->N;i++) assert(I[i]>0);
     if(I[_W->N]>I[0]) { _W->d=I[_W->N]; shift=0; } else _W->d=I[0];
     for(i=0;i<_W->N;i++) {_W->w[i]=I[i+shift]; assert(_W->w[i]<_W->d);} 
     if(FilterFlag) inFILE=NULL; return 1;
}
void Write_WH(Weight *_W, BaHo *_BH, VaHo *_VH, int rc, int tc,
	      PolyPointList *_P, VertexNumList *_V, EqList *_E){
  int i;
  for(i=0;i<_W->N;i++) fprintf(outFILE,"%d ",(int)_W->w[i]);
  fprintf(outFILE,"%d=d M:%d %d ",(int)_W->d,_P->np,_V->nv);  /* PolyData */
  if(rc) fprintf(outFILE,"N:%d %d ",_BH->np,_E->ne); 
  else fprintf(outFILE,"F:%d ",_E->ne); 		 /* END of PolyData */
  if(tc&&(_P->n!=1+_VH->D)) {int D=0, d=_W->d;		/* LG non-geometric */
     fprintf(outFILE,"LG: "); for(i=0;i<_W->N;i++) D+=d-2*_W->w[i]; 
     if(D%d) {int g=Fgcd(D,d); fprintf(outFILE,"c/3=%d/%d\n",D/g,d/g);}
     else { D/=d; assert(D==_VH->D); 
	for(i=0;i<=D;i++) {int j; fprintf(outFILE,"%sH%d:",i?" ":"",i);
	for(j=0;j<=D-i;j++)fprintf(outFILE,"%s%d",j?",":"",(int)_VH->h[i][j]);
	}  fputs("\n",outFILE);}
     return ;
  } if((!tc)&&(!rc)){fprintf(outFILE,"non-transversal\n");return;}
  if(_P->n>2){
    if(tc&&rc) for(i=1;i<_P->n-1;i++) if(_VH->h[1][i]!=_BH->h1[i]) 
      { puts("Vafa!=Batyrev"); for(i=1;i<_P->n-1;i++) printf(
	"V[1,%d]=%d  B[1,%d]=%d\n",i,_VH->h[1][i],i,_BH->h1[i]);exit(0);}
    if(tc)      {	
      fprintf(outFILE,"V:%d",_VH->h[1][1]); 
      for(i=2;i<_P->n-1;i++) fprintf(outFILE,",%d",_VH->h[1][i]);     }
    else if(rc)     {	
      fprintf(outFILE,"H:%d",_BH->h1[1]); 
      for(i=2;i<_P->n-1;i++) fprintf(outFILE,",%d",_BH->h1[i]);     }
    if(6<_P->n) fprintf(outFILE," [???]\n"); 
    else if(tc||rc) /* Euler number */ {	
      int chi=0, *ho=_BH->h1; if(tc) ho=_VH->h[1];
      if(_P->n==3) chi=4+ho[1];
      if(_P->n==4) chi=2*(ho[1]-ho[2]);
      if(_P->n==5) chi=48+6*(ho[1]-ho[2]+ho[3]);
      fprintf(outFILE," [%d]\n",chi);     }	
    else fprintf(outFILE,"\n");} 
  else fprintf(outFILE,"\n");
}
void TEST_LatticeBasis(AmbiLatticeBasis *_B)      /* print AmbiLatticeBasis */
{    int p,a; for(p=0;p<_B->n;p++){
	for(a=0;a<_B->N;a++) printf(" %3d",(int) _B->x[p][a]); puts("");
     }	
}
#ifdef	TEST
void TEST_WeightMakePoints(AmbiPointList *_P)
{    int i,j; static int MaxPoNum; if(_P->np>MaxPoNum) MaxPoNum = _P->np;
     if(_P->np>20)
     {	for(i=0;i<_P->np;i++) 
	{  for(j=0;j<_P->N;j++) printf("%d ",_P->x[i][j]); puts("");}
     }
     else
     {	for(i=0;i<_P->N;i++) 
	{  for(j=0;j<_P->np;j++) printf("%4d",_P->x[j][i]); puts("");}
     }
     printf("PointNum=%d [max=%d]\n",_P->np,MaxPoNum);
}
#endif

void Make_Poly_Points(Weight *_W_in, PolyPointList *_PP)
{    AmbiLatticeBasis B; Weight *_W=_W_in;
     AmbiPointList * _AP = (AmbiPointList *) malloc(sizeof (AmbiPointList));
#ifndef NO_COORD_IMPROVEMENT		/* ==== Perm Coord Improvement ==== */
     Weight Waux=(*_W); _W=&Waux; 
     {	int i,n=_W->N,pi[AMBI_Dmax]; Long M[AMBI_Dmax][AMBI_Dmax],
	    *G[AMBI_Dmax]; for(i=0;i<n;i++) G[i]=M[i];
	Wperm_to_GLZ(_W->w,&n,G,pi);for(i=0;i<n;i++)Waux.w[i]=_W_in->w[pi[i]];
     }
#endif				       /* = End of Perm Coord Improvement = */
     if(_AP==NULL) {printf("Unable to allocate space for _AP\n"); exit(0);}
     WeightLatticeBasis(_W, &B);	/* TEST_LatticeBasis(&B); */
     WeightMakePoints(_W, _AP);	
#ifdef TEST
     puts("\nWeights:");     Write_Weight(_W);  
     puts("AmbiPoints:");    TEST_WeightMakePoints(_AP); 	
     puts("Basis:");	     TEST_LatticeBasis(&B);
     printf("POINT_Num=%d\n",_AP->np);	
#endif
     assert(_AP->np <= POINT_Nmax);   
     ChangeToTrianBasis(_AP, &B, _PP);
#ifdef	TEST
     puts("PolyPoints:");  Print_PPL(_PP);
#endif
     free(_AP); return;
}

int Read_W_PP(Weight *_W, PolyPointList *_PP){
    if (!Read_Weight(_W)) return 0; Make_Poly_Points(_W,_PP); return 1;
}

/*  ==========	   WeightLatticeBasis(Weight *_w, AmbiLatticeBasis *_B)
 *   
 *   M[0]:= (-n1, n0, 0, ...) / gcd(n0,n1);
 *   M[i]:= (0,...,0,g/ng,0,...)- (ni/ng) * egcd.Vout(n0,...,n(i-1),0,...);
 *   	    with g=gcd(n0,...,n[i-1]); ng=gcd(g,ni);
 *									    */
#ifdef TEST
void NormTriangularBasis(AmbiLatticeBasis *_B)
{    int p=_B->n-1, a=_B->N-1;	while(p--) 
     {	int pi=_B->n; 	while(0==_B->x[p][--a]);
	while(--pi > p)
	{   int ai=a+1, r = _B->x[pi][a] / _B->x[p][a];
/**/	    if((_B->x[pi][a] % _B->x[p][a]) < 0) r--;
/**/	    if(r) while(ai--) _B->x[pi][ai] -= r * _B->x[p][ai];
	}
     }
}
void OldWeightLatticeBasis(Weight *_w, AmbiLatticeBasis *_B)	
{    int i, j; 
     Long *V, Vin[W_Nmax], Vout[W_Nmax]; 
     for(i=0; i<_w->N; i++) Vin[i]=_w->w[i];
     V=_B->x[0]; *Vout=Fgcd(Vin[0],Vin[1]); 
     V[0] = -Vin[1] / *Vout; V[1] = *Vin / *Vout; _B->N=_w->N; _B->n=_w->N-1;
     for(j=2; j<_w->N; j++) V[j]=0;
     for(i=2; i<_w->N; i++) {
	int g=REgcd(Vin, &i, Vout), gn=Fgcd(g,_w->w[i]); V=_B->x[i-1]; 
        V[i] = g/gn; g = _w->w[i]/gn;
	for(j=0;j<i;j++) V[j]=-Vout[j]*g; 
	j++; while(j<_w->N) V[j++]=0;
     }
     NormTriangularBasis(_B); /* TEST_LatticeBasis(_B); */
}
#endif
void WeightLatticeBasis(Weight *_w, AmbiLatticeBasis *_B)	
{    Long *B[W_Nmax], E[W_Nmax]; int i; _B->N=_w->N; _B->n=_w->N-1;
     B[0]=E; for(i=0;i<_B->n;i++) B[i+1]=_B->x[i]; W_to_GLZ(_w->w,&(_w->N),B);
}

/*  ==========		WeightMakePoints(Weight *_W, AmbiPointList *_P)
 *
 *	MakePoints: 	X[i]-IP[i] = x[j] * B[j][i] :: i-1<=j<B.n,
 *	     		with  \sum X_i w_i=d  and  0<=X[i].
 *									    */
void WeightMakePoints(Weight *_W, AmbiPointList *_P)
{    int i=_W->N-1, X_max[W_Nmax], d_Rest[W_Nmax], X[W_Nmax+1]={0};
     _P->np=0; _P->N=_W->N; d_Rest[i]=_W->d; X_max[i]=d_Rest[i]/_W->w[i];
     while(i<_W->N) 
     {  if(X[i]>X_max[i]) X[++i]++;
	else if(i) 
	{   d_Rest[i-1]=d_Rest[i]-X[i]*_W->w[i];  X[--i]=0; 
	    X_max[i]=d_Rest[i]/_W->w[i];
	}
	else
	{   int j; 
	    Long *Y; 
	    *d_Rest=d_Rest[1]-X[1]*_W->w[1];
	    if( !(*d_Rest % *(_W->w)) ) 
	    {	if(POINT_Nmax <= _P->np){puts("increase POINT_Nmax");exit(0);}
		Y=_P->x[(_P->np)++];
		*Y= *d_Rest/ *(_W->w); for(j=1;j<_W->N;j++) Y[j]=X[j];
	    }
	    X[1]++; i=1;
	}
     }
}

/*  ==========	 ChangeToTrianBasis(AmbiPointList *, Basis *, PolyPointList *)
 *
 *	Assuming that AmbiLatticeBasis is of triangular form, i.e.:
 *	_B->x[p][a] = 0 for a < p + CoDim  ... a < W_Nmax and p < POLY_Dmax
 *	this solves AP=B.PP for AP (to be precise we must take AP.x-IP, 
 *				    where IP is the unique interior point.)
 *	DP=AP.x-IP; DP[a]=PP[p]*B[p][a] => 
 *		PP[p]*B[p][p+1] = DP[p+1] - sum_{a>p+1} PP[p]*B[p][p+1]
 *									    */
void ChangeToTrianBasis(AmbiPointList *_AP,
			AmbiLatticeBasis *_B, PolyPointList *_PP)
{    int n, ipcount=0, nIP=0;
     if( _AP->N - _B->N) {puts("Dimensions don't match!"); exit(0);}
     else { _PP->n = _B->n; _PP->np = _AP->np; }
     for(n=0; n<_AP->np; n++)
     {	int i, p=1; for(i=0; i < _B->N; i++) if( ! _AP->x[n][i] ) p=0;
	if(p) { ipcount++; nIP=n; }
     }
#ifdef	TEST
     if(ipcount-1) puts("*** Wrong IP-count: take any point as origin! ***");
#endif
     for(n=0; n<_AP->np; n++)
     {	int i, p=_B->n; 
	while(p--) 
	{   int a=p + _B->N - _B->n; _PP->x[n][p]=_AP->x[n][a]-_AP->x[nIP][a];
	    for(i=p+1; i<_B->n; i++) _PP->x[n][p] -= _PP->x[n][i]*_B->x[i][a];
	    if(_PP->x[n][p] % _B->x[p][a]) 
	    { puts("Error in BasisChange!"); exit(0);}
	    else _PP->x[n][p] /= _B->x[p][a];
	}
     }
}
/*  =============		make weights		===========  */
int IfRefWWrite(Weight *W, PolyPointList *P)
{    VertexNumList V; EqList E; Make_Poly_Points(W, P); 
     if(Ref_Check(P,&V,&E)) {Write_Weight(W); fflush(stdout); return 1;} 
     else return 0;
}
void Rec_RefWeights(Weight *W, PolyPointList *P, int g, int sum, int *npp, 
	int *nrp, int n)
{    int wmax=W->d/(W->N-n+1); wmax=min(wmax,W->w[n+1]); 
     wmax=min(wmax,sum-n);
     if(n) for(W->w[n]=wmax;(n+1)*W->w[n]>=sum;W->w[n]--)
	Rec_RefWeights(W,P,Fgcd(g,W->w[n]),sum-W->w[n],npp,nrp,n-1);
     else if(1==Fgcd(g,W->w[0]=sum)) {(*npp)++;if(IfRefWWrite(W,P))(*nrp)++;};
}
void MakeRefWeights(int N, int from_d, int to_d)
{    int npp=0, nrp=0; Weight W; 
     PolyPointList *P = (PolyPointList *) malloc (sizeof(PolyPointList)); 
     W.N=N; assert((N<=W_Nmax)&&(N<POLY_Dmax+2)); assert(P!=NULL);
     for(W.d=from_d;W.d<=to_d;W.d++)
        for(W.w[N-1]=W.d/2; W.d <= N*W.w[N-1]; W.w[N-1]--)
	Rec_RefWeights(&W,P,Fgcd(W.d,W.w[W.N-1]),W.d-W.w[W.N-1],&npp,&nrp,N-2);
     fprintf(outFILE,"#primepartitions=%d #refpolys=%d\n",npp,nrp);exit(0);
}

/*  =============	Landau-Ginzburg-Calculations:		===========  */

void PrintPoCoLi(PoCoLi *P)
{    int i; for(i=0;i<P->n;i++) { int co=abs(P->c[i]); printf("%s ", i ? 
	((P->c[i] > 0) ? "+" : "-" ) : "");if(!i&&(P->c[0]<0))printf("- ");
	if(co!=1) printf("%d ",co); printf("x^%d ",(int)P->e[i]); } 
     puts(i ? "" : " 0");
}
void UnitPoly(PoCoLi *P) { P->n=1; P->e[0]=0; P->c[0]=1; }
void Add_Mono_2_Poly(int e, Pint c, PoCoLi *P)		   /* use bisection */
{    int m=-1, M=P->n;
     if(P->n) if(e <= P->e[P->n-1]) while(M-m>1)
     {	int pos=(M+m)/2; if(e > P->e[pos]) m=pos; else 
	if (e < P->e[pos]) M=pos; else { P->c[pos]+=c; return; }/* e exists */
     }	assert(P->n < P->A);		/* check #(coeff.) of Poly. */
     for(m=P->n++;M<m;m--) {			/* insert new exponent at M */
	P->c[m]=P->c[m-1]; P->e[m]=P->e[m-1]; } P->e[m]=e;P->c[m]=c;
#ifdef	TEST_PP
     {static int M=1; if(P->n/1000000 > M) {printf("#c=%dM ",++M);fflush(0);}}
#endif
}
void Init1_xN(PoCoLi *P,int N)					 /* 1 - x^N */
{    UnitPoly(P); Add_Mono_2_Poly(N,-1,P);
}
void Remove_Zeros(PoCoLi *AB)
{    int i, s=0; for(i=0; i<AB->n; i++)	       /* s=shift: eliminatet zeros */
     {	if(0==AB->c[i]) s++; else 
	if(s) { AB->c[i-s]=AB->c[i];  AB->e[i-s]=AB->e[i]; }
     }	AB->n -= s;
}
int  BottomUpQuot(PoCoLi *N,PoCoLi *D,PoCoLi *Q,PoCoLi *R)     /* Q*D = N-R */
{    int i, e, c, Npos, dD, E;		/* assume D != 0 != N   return R==0 */
     Q->n=R->n=0; if(0>(e=N->e[0]-D->e[0])) { *R=*N; return 0; } 
     dD = D->e[D->n-1]-D->e[0]; E = N->e[N->n-1] - D->e[D->n-1];
     for(Npos=0;Npos<N->n;Npos++)
     {	if(N->e[Npos]<=e+dD)Add_Mono_2_Poly(N->e[Npos],N->c[Npos],R);
	else break;
     }
     while(0==(R->c[0]%D->c[0]))
     {	Add_Mono_2_Poly(e,c=R->c[0]/D->c[0],Q);		      /* next Q.c[] */
	for(i=0;i<D->n;i++)Add_Mono_2_Poly(e+D->e[i],-c*D->c[i],R); /* R-cQ */
	Remove_Zeros(R); if((Npos==N->n)&&(0==R->n)) return 1; /* finished? */
        e = (R->n) ?  R->e[0]-D->e[0] : N->e[Npos]-D->e[0];   /* next Q.e[] */
     	while(Npos<N->n) { if(N->e[Npos]<=e+dD)
	    {   Add_Mono_2_Poly(N->e[Npos],N->c[Npos],R); Npos++;} else break;
	Remove_Zeros(R); if(e > E) return 0;
     	}
     }	while(Npos<N->n){Add_Mono_2_Poly(N->e[Npos],N->c[Npos],R); Npos++;}
     return 0;
}
void PolyProd(PoCoLi *A,PoCoLi *B,PoCoLi *AB)			/* AB = A*B */
{    int i, j, s=A->n+B->n-1; AB->n=0; assert(s<AB->A);
     for(i=0;i<s;i++)
     {	int m = (i<B->n) ? 0 : i+1-B->n,   M = (i<A->n) ? i+1 : A->n; 
	for(j=m;j<M;j++)
	    Add_Mono_2_Poly(A->e[j]+B->e[i-j],A->c[j]*B->c[i-j],AB);
     }	Remove_Zeros(AB);
}
void AllocPoCoLi(PoCoLi *P)			/* allocate e[A] and c[A] */
{    assert(0<P->A); assert( NULL != ( P->e = (int *) malloc( P->A * 
	(sizeof(int)+sizeof(Pint) ) ) ));  P->c = (Pint *) & P->e[P->A];
}
void Free_PoCoLi(PoCoLi *P) { free(P->e); }	/* free P.e and P.c */
void PoincarePoly(int N, int *w, int d, PoCoLi *P, PoCoLi *Z,PoCoLi *R)
{    int i, e[2]; Pint c[2]; PoCoLi B, *In=Z,*Out=P,*aux;
     if(N==0) {UnitPoly(P); return;} 
     B.e=e; B.c=c; B.A=B.n=2; e[0]=0; c[0]=1; c[1]=-1; Init1_xN(In,d - w[0]);
     for(i=1;i<N;i++)
     {	e[1]=d-w[i]; PolyProd(In,&B,Out); aux=Out; Out=In; In=aux;
     }  /* printf("\nN =");PrintPoCoLi(Z); */
     while(i--)
     {	e[1]=w[i]; assert(BottomUpQuot(In,&B,Out,R)); aux=Out; Out=In; In=aux;
     }	/* printf("Q =");PrintPoCoLi(P); */	assert((R->n)==0); 
#ifdef	TEST_PD
     	{  int M=P->n-1, I=(M+1)/2, E=P->e[P->n-1];
	for(i=0;i<I;i++) {assert(P->e[i]==E-P->e[M-i]); 
	assert(P->c[i]==P->c[M-i]); }}
#endif
}

VaHo Calc_VaHo(Weight *W) {    
     int i, j, k, D=0, w[W_Nmax], N=W->N, d=W->d; VaHo V;
     PoCoLi A,B,C, *Z=&A, *R=&B, *P=&C; 
     for(i=0;i<N;i++) { w[i]=W->w[i]; D+=d-2*w[i]; }
     if(D % d) {V.D=0; return V;} else V.D = (D/=d);
     for(i=0;i<N;i++)for(j=i+1;j<N;j++)if(w[j]<w[i]) swap(&w[j],&w[j]);
#ifndef	COEFF_Nmax
#define	COEFF_Nmax	(d*D+2*N)		/* 22999000 for W_Nmax == 6 */
#endif
     P->A=Z->A=COEFF_Nmax; R->A=w[N-1]+1;	
     AllocPoCoLi(P); AllocPoCoLi(Z); AllocPoCoLi(R);  
     PoincarePoly(N,w,d,P,Z,R);		
	assert(D*d==P->e[P->n-1]);
     {  int n=P->n, cM=0; long long sum=0,num=1,den=1; /* check sum = P(0) */
	for(i=0;i<n;i++) {Pint co=P->c[i]; assert(co>0); 
	    if(co>cM) cM=co; sum+=co; }
	for(i=0;i<W->N;i++){	num*=W->d-W->w[i]; den*=W->w[i];
	   {long long g=LFgcd(num,den); num/=g; den/=g; }} 
#ifdef	TEST_PP
	if(P->n<99) {printf("PP =");PrintPoCoLi(P);} else printf(
	   "#(Exp,Co)=%d  Exp<=%d  Coeff<=%d  sum=%lld\n",n,P->e[n-1],cM,sum);
#endif
	if(den==1) assert(num==sum);
	else { printf("sum=%lld  test=%lld/%lld \n",sum,num,den);}
	}
     for(i=0;i<=D;i++) for(j=0;j<=D;j++) V.h[i][j] = 0;
     for(i=0;i<P->n;i++)if(P->e[i] % d==0) {j=P->e[i]/d; V.h[D-j][j]=P->c[i];}
#ifdef	TEST_PP
     for(i=0;i<=D;i++){for(j=0;j<=D;j++)printf("%6d ",(int)V.h[i][j]);
	puts("= V-pri");}fflush(0);
#endif
     for(k=1;k<d;k++)					/* k-twisted sector */
     {	int th[W_Nmax], Tmt, et=0, eT=0, Deff=0, n=0;
	for(i=0;i<N;i++)
	{   if(th[i]=((long long) k * W->w[i]) % (long long)d)
	    {	et+=th[i]-W->w[i];eT+=W->d-th[i]-W->w[i];}
	    else { Deff+=W->d-2*W->w[i]; w[n++]=W->w[i];}    /* effective w */
	}
	if(0 == ((Tmt=eT-et) % d))
	{   for(i=0;i<n;i++)for(j=i+1;j<n;j++)if(w[j]<w[i]) swap(&w[j],&w[j]);
	    PoincarePoly(n,w,d,P,Z,R); assert(Deff==P->e[P->n-1]); Tmt/=d;
	    for(i=0;i<P->n;i++) 
	    {	j=et+P->e[i]; 
		if(j % d==0) {j/=d; if(j+Tmt>=0)V.h[D-j][j+Tmt]+=P->c[i];}
	    }
	}
     }	Free_PoCoLi(P); Free_PoCoLi(Z); Free_PoCoLi(R); 
#ifdef	TEST_PP
     for(i=0;i<=D;i++){for(j=0;j<=D;j++)printf("%6d ",V.h[i][j]);puts("= V");}
#endif
	return V;
}


/*  =============	     Trans_Check(Weight W)		==========  */

int symcheck(int sum, int link, int *mon, Weight W, int *mask){
/* symcheck returns 1 if there is a monomial mon in
 * the variables indicated by link whose total weight is sum and which
 * transforms under the k'th symmetry with a phase sum[k];
 * if X_i occurs in the monomial then mon[i] is set to 1.                   */
   int i, j, newsum, newlink;

   for (i=0;i<W.N;i++) if (link&mask[i]){
      if (!(sum % W.w[i])){
        *mon=*mon|mask[i]; 
        return 1;};   
      newlink=link-mask[i];
      for (j=0;j*W.w[i]<sum;j++){
        newsum=sum-j*W.w[i];
        if (symcheck(newsum,newlink,mon,W,mask)) {
          if (j) *mon=*mon|mask[i];
          return 1; };   };   };
   return 0;
}
void Init_Trans_Check(int *mask, int **targets, int **mighty)
{    int i, j, maxPN=1<<W_Nmax; mask[0]=1;		  /* maxPN=2^W_Nmax */
     for(i=1;i<=W_Nmax;i++)mask[i]=2*mask[i-1]; /* mask={1,2,4,8,16,32,...} */

     assert(maxPN==mask[W_Nmax]);/* maximum number of pointers at one point */
     assert( NULL != (*targets = (int *) malloc(maxPN * sizeof(int))) );
     assert( NULL != (*mighty  = (int *) malloc(maxPN * sizeof(int))) );

     for (i=0;i<W_Nmax;i++) for (j=mask[i];j<mask[i+1];j++)/* make mighty[] */
       (*mighty)[j]=(*mighty)[j-mask[i]]+1;		       
}

int Trans_Check(Weight W){  /* returns 1 if non-degenerate potential exists */
   int i, j, k, mon;   			/* j represents the link!!!        */
   static int mask[1+W_Nmax], *targets, *mighty; assert(W.N<=W_Nmax);
   if(targets==NULL) Init_Trans_Check(mask,&targets,&mighty); targets[0]=0;
   for (i=0;i<W.N;i++) for (j=mask[i];j<mask[i+1];j++){
     targets[j]=0;
     for (k=0;k<=i;k++) targets[j]=targets[j]|targets[j&~mask[k]];
     for (k=0;(k<W.N)&&(mighty[targets[j]]<mighty[j]);k++) 
       if (!(mask[k]&targets[j])) if (symcheck(W.d-W.w[k],j,&mon,W,mask)) 
         targets[j]=targets[j]|mask[k];
     if (mighty[targets[j]]<mighty[j]) return 0;}
   return 1;
}
