/*  ======================================================================  */
/*  ==========                                                  ==========  */
/*  ==========                P O L Y . C                       ==========  */
/*  ==========                                                  ==========  */
/*  ==========                 April 2002                       ==========  */
/*  ==========                                                  ==========  */
/*  ======================================================================  */

/*
 *	Coord   get coordinates by reading them or converting weight input
 *	Rat	rational functions
 *	Vertex	computes Vertices and Faces
 *      Polynf	normal form and symmetries
 *	-> class.x
 */

#include "Global.h"
#include "LG.h"

FILE *inFILE, *outFILE;

void  PrintUsage(char *c){
  int i;
#define	PUli	19
#define	PUitems	2*PUli
  char opt_string[PUitems][40]={
  "h  print this information           ",
  "f  use as filter                    ",
  "g  general output:                  ",
  "   P reflexive: numbers of (dual)   ",
  "     points/vertices, Hodge numbers ",
  "   P not reflexive: numbers of      ",
  "     points, vertices, equations    ",
  "p  points of P                      ",
  "v  vertices of P                    ",
  "e  equations of P/vertices of P-dual",
  "m  pairing matrix between vertices  ",
  "     and equations                  ",
  "d  points of P-dual                 ",
  "     (only if P reflexive)          ",
  "a  all of the above except h,f      ",
  "l  LG-`Hodge numbers' from single   ",
  "     weight input                   ",
  "r  ignore non-reflexive input       ",
  "D  dual polytope as input (ref only)",
  "n  do not complete polytope or      ",
  "     calculate Hodge numbers        ",
  "i  incidence information            ",
  "s  check for span property          ",
  "     (only if P from CWS)           ",
  "I  check for IP property            ",
  "S  number of symmetries             ",
  "T  upper triangular form	       ",
  "N  normal form                      ",
  "t  traced normal form computation   ",
  "V  IP simplices among vertices of P*",
  "P  IP simplices among points of P*  ",
  "     (with 1<=codim<=# when # is set)",
  "Z  lattice quotients for IP simplices",
  "#  #=1,2,3  fibers spanned by IP    ",
  "     simplices with codim<=#        ",
  "## ##=11,22,33,(12,23): all (fibered)",
  "     fibers with specified codim(s) ",
  "   when combined: ### = (##)#       "};

  printf("This is ``%s'':  computing data of a polytope P\n",c);
  printf("Usage:   %s [-<Option-string>] [in-file [out-file]]\n", c);
  printf("Options (concatenate any number of them into <Option-string>):\n");
  for (i=0;i<PUli;i++) printf("  %s | %s\n",opt_string[i],opt_string[i+PUli]);
  puts("Input:    degrees and weights `d1 w11 w12 ... d2 w21 w22 ...'");
  puts("          or `d np' or `np d' (d=Dimension, np=#[points]) and");
  puts("              (after newline) np*d coordinates");
  puts("Output:   as specified by options");
}

main (int narg, char* fn[]){
  int n=0, k, FilterFlag=0, lg=0, s=0, i=0, m=0, p=0, v=0, e=0, d=0, t=0,
    S=0, N=0, I=0, r=0, nc=0, g=0, D=0, IP, R, Tr, T=0, PS=0, VS=0, CD=0, ZS=1;
  char c;
  CWS *CW=(CWS *) malloc(sizeof(CWS));
  Weight W;
  VertexNumList V;
  EqList *E = (EqList *) malloc(sizeof(EqList));
  EqList *DE = (EqList *) malloc(sizeof(EqList));
  BaHo BH;
  VaHo VH;
  PolyPointList *_P = (PolyPointList *) malloc(sizeof(PolyPointList)),
               *_DP = (PolyPointList *) malloc(sizeof(PolyPointList));
  FaceInfo *FI;
  PairMat *PM = (PairMat *) malloc(sizeof(PairMat)),
         *DPM = (PairMat *) malloc(sizeof(PairMat));


  if((CW==NULL)||(E==NULL)||(_P==NULL)||(_DP==NULL)||(PM==NULL)||(DPM==NULL)) {
    puts("Allocation failure: Reduce dimensions!"); exit(0);}
  CW->nw=0;

  while(narg > ++n) {
    if(fn[n][0]!='-') break;
    k=0;
    while ((c=fn[n][++k])!='\0'){
      if(c=='h') { PrintUsage(fn[0]); exit(0);}
      if(c=='f') FilterFlag=1;
      if(c=='g') g=1;
      if(c=='l') lg=1;
      if(c=='s') s=1;
      if(c=='i') i=1;
      if(c=='I') I=1;
      if(c=='m') m=1;
      if(c=='p') p=1;
      if(c=='v') v=1;
      if(c=='e') e=1;
      if(c=='d') d=1;
      if(c=='t') t=1;
      if(c=='S') S=1;
      if(c=='N') N=1;
      if(c=='T') T=1;
      if(c=='r') r=1;
      if(c=='n') nc=1;
      if(c=='P') PS=1;
      if(c=='V') VS=1;
      if(c=='Z') ZS=-1;
      if(c=='D') D=1;
      if(('0'<c)&&(c<='9')) CD=10*CD+c-'0';
      if(c=='a') {g=1; m=1; p=1; v=1; e=1; d=1; }}}
  n--;
  if(s+i+I+m+p+v+e+d+t+S+N+T+r+PS+VS+CD==0) g=1;
  if(FilterFlag) {inFILE=NULL; outFILE=stdout;}
  else {
    if (narg > ++n)  inFILE=fopen(fn[n],"r");
    else inFILE=stdin;
    if (inFILE==NULL){printf("Input file %s not found!\n",fn[n]);exit(0);}
    if (narg > ++n) outFILE=fopen(fn[n],"w");
    else outFILE=stdout;     }

  if(i){
    FI=(FaceInfo *) malloc(sizeof(FaceInfo));
    if (FI==NULL) {puts("Unable to allocate space for FaceInfo FI"); exit(0);}}


  while(lg ? Read_W_PP(&W,_P) : Read_CWS_PP(CW,_P)) {
    if(T) Make_Poly_UTriang(_P);
    R=0;
    if (IP=Find_Equations(_P,&V,E)){
      if (D){
	int k;
	*_DP=*_P;
	R=EL_to_PPL(E, _P, &_P->n);
	VNL_to_DEL(_DP, &V, E);
	for (k=0;k<_P->np;k++) V.v[k]=k;
	V.nv=_P->np;      }
      else R=EL_to_PPL(E, _DP, &_P->n);}
    if (D&&!R) {fprintf(outFILE,"Input not reflexive!\n"); continue;}
    if (r&&!R) continue;
    Sort_VL(&V);
    Make_VEPM(_P,&V,E,*PM);
    if(!nc) {
      if(D||!(lg||CW->nw)) Complete_Poly(*PM,E,V.nv,_P);
      if(R&&!(D&&(lg||CW->nw))) {
	assert(Transpose_PM(*PM, *DPM, V.nv, E->ne));
	VNL_to_DEL(_P, &V, DE);
	Complete_Poly(*DPM,DE,E->ne,_DP);}}
    if(g){
      if(!R||nc) {BH.mp=_P->np; BH.mv=V.nv; BH.np=0; BH.nv=E->ne;}
      else RC_Calc_BaHo(_P,&V,E,_DP,&BH);
      if(lg) {
	if (Tr=Trans_Check(W)) VH=Calc_VaHo(&W);
	Write_WH(&W, &BH, &VH, R, Tr, _P, &V, E); }
      else Print_CWH(CW, &BH); }
    if(s&&CW->nw) if(!Span_Check(E,&(CW->B),&_P->n))
      fprintf(outFILE,"No Span\n");
    if(I && !IP) fprintf(outFILE,"No IP\n");
    if(p) Print_PPL(_P,"Points of P");
    if(v) Print_VL(_P, &V, "Vertices of P");
    if(e) Print_EL(E, &_P->n, R,
          (R ? "Vertices of P-dual <-> Equations of P" : "Equations of P"));
    if(i){Make_Incidence(_P,&V,E,FI); Print_FaceInfo(_P->n,FI);}
    if(m) Print_Matrix(*PM, E->ne, V.nv,
		       "Pairing matrix of vertices and equations of P");
    if(d&&(_DP->np>E->ne)) Print_PPL(_DP, "Points of P-dual");
    if(S||N||t){
      int SymNum, VPMSymNum, V_perm[SYM_Nmax][VERT_Nmax];
      Long NF[POLY_Dmax][VERT_Nmax]; /* printf("=%d",t); fflush(0); */
      VPMSymNum=Make_Poly_Sym_NF(_P, &V, E, &SymNum, V_perm, NF, t);
      if (S) fprintf(outFILE,"#GL(Z,%d)-Symmetries=%d, #VPM-Symmetries=%d\n",
		     _P->n, SymNum, VPMSymNum);
      if (N) Print_Matrix(NF, _P->n, V.nv,"Normal form of vertices of P");}
    if(R&&(PS||VS||CD)) IP_Simplices(_DP, (!D)*E->ne, PS*ZS, VS*ZS, CD);
    fflush(outFILE);     }
}

